Function Expand-TarGZ {
    [CmdletBinding()]
    Param(
        [string] $inFile,
        [string] $outFolder = ($inFile -replace '\.tar.gz$','')
    )

    $tarFile = Expand-GZ -InFile $inFile
    if (-not $tarFile) {
        Write-Error "Cannot unpack gzip $inFile into $tarFile"
        Exit
    }

    if (-not (Get-Command Expand-7Zip -ErrorAction Ignore)) {
        Install-Package -Scope CurrentUser -Force 7Zip4PowerShell > $null
    }
    Expand-7Zip $tarFile $outFolder

    Remove-Item $tarFile -Recurse -Force
    return $outFolder
}

Function Expand-GZ([string] $inFile) {
    $outFile = ($inFile -replace '\.gz$','')
    if (Test-Path -Path $outFile -PathType leaf) {
        Remove-Item -Path $outFile -Force
    }
    $BUFFER_SIZE = 1024
    $inputStream = New-Object System.IO.FileStream $inFile, ([IO.FileMode]::Open), ([IO.FileAccess]::Read), ([IO.FileShare]::Read)
    $outputStream = New-Object System.IO.FileStream $outFile, ([IO.FileMode]::Create), ([IO.FileAccess]::Write), ([IO.FileShare]::None)
    $gzipStream = New-Object System.IO.Compression.GzipStream $inputStream, ([IO.Compression.CompressionMode]::Decompress)

    $buffer = New-Object byte[]($BUFFER_SIZE)
    while($true) {
        $read = $gzipstream.Read($buffer, 0, $BUFFER_SIZE)
        if ($read -le 0){break}
        $outputStream.Write($buffer, 0, $read)
    }

    $gzipStream.Close()
    $outputStream.Close()
    $inputStream.Close()
    return $outFile
}