﻿# Install concept product
function Install-Product {
    [CmdletBinding()]
    Param(
        [string] $product,
        [string] $distr,
        [string] $localTemplates,
        [string] $localTools
    )

    if ($product -eq 'ExcelHelper') {
        return InstallExcelAddinOptional $distr 'CONCEPT.xlam' $localTools
    } elseif ($product -eq 'WordHelper') {
        return $(InstallWordAddinOptional $distr 'CONCEPT.dotm' $localTools -and
                InstallUserModel $distr 'banned-words.txt')
    } elseif ($product -eq 'Concept-Maket') {
        return InstallWordAddinOptional $distr '_Maket.dotm' $localTools
    } elseif ($product -eq 'Concept-Reports') {
        return InstallExcelAddinOptional $distr "ConceptReport.xlam" $localTools
    } elseif ($product -eq 'Concept-Hierarchy') {
        return InstallTemplate $distr '30 Иерархизатор.vstm' $localTemplates
    } elseif ($product -eq 'Concept-Blocks') {
        return $(InstallTemplate $distr '20 Концепт-Блоки.vstm' $localTemplates -and
            InstallTemplate $distr 'Технологии\Блоки-Excel.xltx' $localTemplates -and
            InstallTemplate $distr 'Технологии\Блоки-Word.dotx' $localTemplates)
    } elseif ($product -eq 'Concept-Defs') {
        return $(InstallTemplate $distr '22 Карта понятий.vstm' $localTemplates -and
            InstallTemplate $distr 'Технологии\Определения-Excel.xltx' $localTemplates)
    } elseif ($product -eq 'Concept-NPA') {
        Write-Host "Copying file !Реестр НПА.xlsm"
        Copy-Item -Path "$distr\data\!Реестр НПА.xlsm" -Destination "$localTools\optional\" -Force
        return $(InstallTemplate $distr '60 НПА UI.xltm' $localTemplates -and
            InstallTemplate $distr '21 Схема Реестра НПА.vstm' $localTemplates -and
            InstallWordAddinOptional $distr '_Concept-NPA.dotm' $localTools)
    } elseif ($product -eq 'Concept-Mining') {
        return $(InstallTemplate $distr '55 Майнинг.xltm' $localTemplates -and
            InstallWordAddinOptional $distr 'Parsers.dotm' $localTools -and
            InstallLocalModel $distr 'ActionVerbs.txt' $localTools)
    } elseif ($product -eq 'Concept-Markup') {
        return $(InstallTemplate $distr 'Разметка' $localTemplates -and
            InstallWordAddinOptional $distr 'MARKUP.dotm' $localTools)
    }
    return $false
}

function TryCopy($source, $destination) {
    try {
        Copy-Item -Path $source -Destination $destination -Force -Recurse -ErrorAction Stop
        return $true
    } catch {
        return $false
    }
}

function InstallUserModel($distr, $fileName) {
    $source = "$distr\models\$fileName"
    $destination = "$(Get-ConceptLocal)\models\"
    Write-Host "Update user model $fileName at $destination"
    return TryCopy $source $destination
}

function InstallLocalModel($distr, $fileName, $localTools) {
    $source = "$distr\models\$fileName"
    $destination = "$localTools\models\"
    Write-Host "Copying model file $fileName to $destination"
    return TryCopy $source $destination
}

function InstallTemplate($distr, $fileName, $templates) {
    $source = "$distr\data\Templates\$fileName"
    $destination = "$templates\"
    Write-Host "Copying template $fileName to $destination"
    return TryCopy $source $destination
}

function InstallExcelAddin($distr, $fileName) {
    $source = "$distr\data\Add-ins\Excel\$fileName"
    $destination = "$($Env:APPDATA)\Microsoft\Excel\XLSTART\"
    Write-Host "Copying addin $fileName to $destination"
    return TryCopy $source $destination
}

function InstallExcelAddinOptional($distr, $fileName, $localTools) {
    $source = "$distr\data\Add-ins\Excel\$fileName"
    
    $destination1 = "$localTools\optional\"
    Write-Host "Copying optional addin $fileName to $destination1"
    if (-not (TryCopy $source $destination1)) {
        return $false
    }

    $destination2 = "$($Env:APPDATA)\Microsoft\Excel\XLSTART\"
    if (Test-Path -Path "$destination2\$fileName" -PathType Leaf) {
        Write-Host "Copying optional addin $fileName to $destination2"
        return TryCopy $source $destination2
    }
    return $true
}

function InstallWordAddin($distr, $fileName) {
    $source = "$distr\data\Add-ins\Word\$fileName"
    $destination = "$($Env:APPDATA)\Microsoft\Word\STARTUP\"
    Write-Host "Copying addin $fileName to $destination"
    return TryCopy $source $destination
}

function InstallWordAddinOptional($distr, $fileName, $localTools) {
    $source = "$distr\data\Add-ins\Word\$fileName"
    
    $destination1 = "$localTools\optional\"
    Write-Host "Copying optional addin $fileName to $destination1"
    if (-not (TryCopy $source $destination1)) {
        return $false
    }

    $destination2 = "$($Env:APPDATA)\Microsoft\Word\STARTUP\"
    if (Test-Path -Path "$destination2\$fileName" -PathType Leaf) {
        Write-Host "Copying optional addin $fileName to $destination2"
        return TryCopy $source $destination2
    }
    return $true
}