# Push python module to Concept server and distr
function Push-PythonPackage {
    [CmdletBinding()] 
    Param(
    [Parameter(Mandatory=$true)]
        [string] $packageName,
    [Parameter(Mandatory=$true)]
        [string] $packageRoot,
        [bool] $pushServer = $true,
        [bool] $pushDistr = $true
    )

    . "$PSScriptRoot\Get-ServerDistr.ps1"
    . "$PSScriptRoot\Get-ExchangePath.ps1"
    
    if ($pushServer) {
        UpdatePythonIn -PackageName $packageName -PackageRoot $packageRoot -ServerRoot "$(Get-ServerDistr)\src"
    }
    if ($pushDistr) {
        UpdatePythonIn -PackageName $packageName -PackageRoot $packageRoot -ServerRoot "$(Get-ExchangePath)\ConceptDistr\src"
    }
}

function UpdatePythonIn($packageName, $packageRoot, $serverRoot) {
    . "$PSScriptRoot\Expand-TarGZ.ps1"

    $targz = Get-Childitem -Path "$packageRoot\output\$packageName\*.tar.gz" -Name
    if (-not $targz) {
        Write-Error "Cannot find unique source distribution in $packageRoot\output\$packageName\"
        Exit
    }
    $targz = "$packageRoot\output\$packageName\$targz"

    $destination = "$serverRoot\$packageName"
    if (Test-Path -Path $destination) {
        Remove-Item $destination -Recurse -Force
    }

    $unpacked = Expand-TarGZ -InFile $targz
    if (-not $unpacked) {
        Write-Error "Cannot unpack $targz"
        Exit
    }

    Move-Item -Path  "$unpacked\$packageName-*\*" -Destination "$unpacked"
    Remove-Item -Path "$unpacked\$packageName-*"

    Move-Item -Path "$unpacked" -Destination "$destination"
}