# Prepare and deploy distribution files for standalone distributive
Set-StrictMode -Version 3.0
Set-Variable -Name PYTHON_DOWNLOAD_VERSION -Value '3.12.2'
Set-Variable -Name MODULE_NAME -Value 'ConceptDeploy'

function DeployDistributives {
    & .\CD_GenerateManifest.ps1
    Import-Module -Name ".\$MODULE_NAME" -Force

    Write-Host "Preparing python environment clone..."
    $preparedEnvironment = PrepareEnvironemnt

    Write-Host "Creating python packages archive from local environment..."
    $envZip = 'C:\Tools\Python312-venv.zip'
    Compress-Archive -Path "$preparedEnvironment\*" -DestinationPath $envZip -Force
    Remove-Item $preparedEnvironment -Recurse -Force

    Write-Host "Downloading python $PYTHON_DOWNLOAD_VERSION installer..."
    $pythonDistr = DowloadPython $PYTHON_DOWNLOAD_VERSION

    $destination = "$(Get-ExchangePath)\ConceptDistr\distr"
    if (-not (Test-Path -Path $destination)) {
        New-Item -Path $destination -ItemType Directory -Force | Out-Null
    }

    Write-Host "Moving files to destination... $destination"
    Copy-Item -Path $pythonDistr -Destination "$destination\" -Force
    Copy-Item -Path $envZip -Destination "$destination\" -Force

    Remove-Item $pythonDistr -Force
    Remove-Item $envZip -Force
}

function PrepareEnvironemnt {
    $coreFiles = 'activate', 'activate.bat', 'Activate.ps1', 'deactivate.bat', 'python.exe', 'pythonw.exe', 'pythonw_d.exe', 'python_d.exe'
    $tools = 'C:\Tools'
    $original = "$tools\Python312-venv"
    $envClone = "$tools\Python312-cln"
    if (-not (Test-Path -Path $original)) {
        Exit 1
    }    
    if (Test-Path -Path $envClone) {
        Remove-Item $envClone -Recurse -Force
    }
    New-Item -Path $envClone -ItemType Directory -Force | Out-Null
    Copy-Item -Path "$original\Include" -Destination "$envClone\Include" -Recurse
    Copy-Item -Path "$original\Lib" -Destination "$envClone\Lib" -Recurse
    Copy-Item -Path "$original\Scripts" -Destination "$envClone\Scripts" -Recurse

    Get-ChildItem $envClone -Recurse | Where-Object{$_.Name.EndsWith('.pyc')} | Remove-Item
    Get-ChildItem -Path "$envClone\Scripts\*" -Recurse -Include $coreFiles | Remove-Item

    return $envClone
}

function DowloadPython($version) {
    $pythonUrl = "https://www.python.org/ftp/python/$version/python-$version-amd64.exe"
    $pythonTemp = "$Env:TEMP\python-$version-amd64.exe"
    try {
        (New-Object System.Net.WebClient).DownloadFile($pythonUrl, $pythonTemp)
    } catch [System.Net.WebException] { 
        Write-Error "An exception was caught when trying to download Python: $($_.Exception.Message)"
        Exit
    } 
    return $pythonTemp
}

DeployDistributives