# Uninstall Concept products

$localHome = 'C:\Tools'
$desktopPath = [Environment]::GetFolderPath('Desktop')
$Env:PSModulePath += ";${localHome}"

function UninstallConceptProducts {
    [CmdletBinding()]
    Param(
    )
    Set-StrictMode -Version 3.0
    $InformationPreference="Continue"

    StartLog

    Write-Host "Starting uninstallation procedure for Concept technologies`n" -ForegroundColor DarkGreen

    PrepareExternalProcesses

    if (-not (Get-Module -ListAvailable ConceptDeploy)) {
        Write-Host "Missing ConceptDeploy library"
        Exit 1
    }
    Import-Module -Name ConceptDeploy

    Write-Host "`nRemoving  Concept Products" -ForegroundColor DarkGreen
    $localJSON = Open-DistrManifest "$localHome\distribution_manifest.json"
    foreach ($product in $localJSON.Keys) {
        Disable-Product $product | Out-Null
        Write-Host "Product $product has been removed`n" -ForegroundColor Gray
    }
    TryRemove "$localHome\optional\"
    TryRemove "$localHome\models\"
    TryRemove "$localHome\distribution_manifest.json"

    Write-Host "`nRemoving  user specific files" -ForegroundColor DarkGreen
    TryRemove "$(Get-ConceptLocal)\"

    $templates = GetTemplatesFolder
    $fileList = Get-Content "$localHome\script\templates_list.txt"
    Write-Host "Remove templates at $templates" -ForegroundColor DarkGreen
    foreach ($file in $fileList) {
        if ($file -ne 'Normal.dotm' -and $file -ne 'NormalEmail.dotm') {
            TryRemove "$templates\$file"
        }
    }
    TryRemove "$localHome\script\templates_list.txt"

    Write-Host "`nRemoving core files"
    TryRemove "$localHome\script\UpdateConceptProducts.ps1"
    TryRemove "$localHome\script\UninstallConceptProducts.ps1"
    TryRemove "$localHome\script\uninstall-concept.bat"
    TryRemove "$localHome\Python312-venv\"
    TryRemove "$localHome\dll\"

    Write-Host "`nRemoving  ConceptDeploy library" -ForegroundColor DarkGreen
    Remove-Module -Name ConceptDeploy
    TryRemove "${localHome}\ConceptDeploy"

    Write-Host "`nRemoving desktop shortcuts" -ForegroundColor DarkGreen
    TryRemove "${desktopPath}\Everything - поиск по имени.lnk"
    TryRemove "${desktopPath}\DocFetcher - поиск по содержимому.lnk"
    TryRemove "${desktopPath}\Double Commander.lnk"

    Write-Host "`nUninstallation complete" -ForegroundColor Green

    StopLog
}

function StartLog {
    $ErrorActionPreference="SilentlyContinue"
    Stop-Transcript | Out-Null
    $ErrorActionPreference = "Continue"

    $dateSuffix = (Get-Date -UFormat "%Y%m%d_%I-%M-%S").ToString()
    $logFile = "${PSScriptRoot}\logs\$($dateSuffix)_CP-uninstall_$($Env:USERNAME).txt"
    Start-Transcript -Path $logFile -IncludeInvocationHeader
}

function StopLog {
    Stop-Transcript
}

function CheckProcess($processName) {
    $app = Get-Process $processName -ErrorAction SilentlyContinue
    return -not ($null -eq $app)
}

function TryRemove($path) {
    if (Test-Path -Path $path) {
        Write-Host "Removing $path..." -ForegroundColor Gray
        Remove-Item -Path $path -Recurse -Force
    } else {
        Write-Host "Failed to locate $path..." -ForegroundColor Red
    }
}

function PrepareExternalProcesses {
    Write-Host "Close blocking applications..." -ForegroundColor DarkGreen
    if ($(CheckProcess WINWORD)) {
        Stop-Process -Name WINWORD -Force
    }
    if ($(CheckProcess EXCEL)) {
        Stop-Process -Name EXCEL -Force
    }
    if ($(CheckProcess VISIO)) {
        Stop-Process -Name VISIO -Force
    }
    if ($(CheckProcess Exteor)) {
        Stop-Process -Name Exteor -Force
    }
    Write-Host "Word running: $(CheckProcess WINWORD)"
    Write-Host "Excel running: $(CheckProcess EXCEL)"
    Write-Host "Visio running: $(CheckProcess VISIO)"
    Write-Host "Exteor running: $(CheckProcess Exteor)"
}

function GetTemplatesFolder {
    $officeRK = 'HKCU:\SOFTWARE\Microsoft\Office\16.0'
    $defaultTemplates = "$($Env:APPDATA)\Microsoft\Templates"

    $templates = Get-ItemPropertyValue -Path "$officeRK\Word\Options" -Name 'PersonalTemplates' -ErrorAction 'SilentlyContinue'
    if (!$templates -or [string]::IsNullOrWhiteSpace($templates)) {
        $templates = $defaultTemplates
    }
    return $templates
}

# UninstallConceptProducts